
# Create directory for saving results within output folder
dir.create(file.path(OutFolder, imageDirArray[TILE,1]), showWarnings = FALSE)

########################################################################################
Cloud<-Snow<-WT<-Shadow<-idplcd<-mask<-NDVI<-NDSI<-whiteness<-idlnd<-raster(im_B2)
mask[] <- !is.na(im_B2[])
NDVI[]<-(im_B5[]-im_B4[])/(im_B5[]+im_B4[])
NDSI[]<-(im_B3[]-im_B6[])/(im_B3[]+im_B6[])

NDVI[which((im_B5[]+im_B4[])==0)]<-0.01
NDSI[which((im_B3[]+im_B6[])==0)]<-0.01
#plot(NDVI,main='NDVI',zlim=c(-1,1))

# saturation in the three visible bands
satu_Bv=(B1Satu[]+B2Satu[]+B3Satu[]>=1)
rm(B1Satu)
gc()

# Basic Cloud Test
idplcd[]<-((NDSI[]<0.8)&(NDVI[]<0.8)&(im_B7[]>300)&(im_B11[]<2700))
plot(idplcd,main='Basic Cloud Test')

# Snow test
# It takes every snow pixels including snow pixel under thin clouds or icy clouds
Snow[] <- ((NDSI[]>0.15)&(im_B5[]>1100)&(im_B3[]>1000))
setwd(file.path(OutFolder, imageDirArray[TILE,1]))

writeRaster(Snow, filename="SnowMask.tif", format="GTiff", overwrite=TRUE)   # write that raster to file
#plot(Snow,main="Snow Test",zlim=c(0,1))
#Snow(mask==0)=255;
rm(Snow);gc()

# Water test
# Zhe's water test (works over thin cloud)
WT[] <- ((NDVI[]<0.01)&(im_B5[]<1100))|((NDVI[]<0.1)&(NDVI[]>0)&(im_B5[]<500))
#plot(WT,main="Water Test",zlim=c(0,1))
gc()
#WT(mask==0)=255;

# Whiteness test
# visible bands flatness (sum(abs)/mean < 0.6 => brigt and dark cloud )
visimean=(im_B2[]+im_B3[]+im_B4[])/3
whiteness[]<-(abs((im_B2[]-visimean)/visimean)+abs((im_B3[]-visimean)/visimean)+abs((im_B4[]-visimean)/visimean))
rm(visimean); gc()

# update idplcd
whiteness[] <- whiteness[]*(satu_Bv!=1) # If one visible is saturated whiteness == 0
idplcd[] <- idplcd[]*(whiteness[]<0.7)
plot(idplcd,main='After Whiteness Test')

# Haze test
HOT=im_B2[]-0.5*im_B4[]-800 # Haze test
idplcd[] <- idplcd[]*(HOT>0|satu_Bv==1) # need to find thick warm cloud
rm(HOT,satu_Bv);gc()
plot(idplcd,main='After Haze Test')

# Ratio 4/5>0.75 cloud test
Ratio4_5=im_B5[]/im_B6[]
idplcd[] <- idplcd[]*(Ratio4_5>0.75)
rm(Ratio4_5); gc()
plot(idplcd,main='After 4/5 Ratio Test')

# Cirrus tests from Landsat 8
Thin_prob = im_B9[]/400
rm(im_B9); gc()
Thin_prob[Thin_prob<0]=0
idplcd[] <- idplcd[]|(Thin_prob > 0.1)   # Threshold of 0.01 in Zhu & Woodcock (2014)?
plot(idplcd,main='After Cirrus Test')

# constants
l_pt=0.175   # low percent
h_pt=1-l_pt  # high percent

# (temperature & snow test )
# test whether use thermal or not
idclr=(1-idplcd[])*mask[]
ptm=100*sum(idclr[],na.rm=T)/sum(mask[],na.rm=T) # percent of clear pixel
idlnd[]=(idclr[])*(1-WT[])
idwt =(idclr[])*(WT[])
rm(idclr); gc()

if (ptm <= 0.1){ # no thermal test => meanless for snow detection (0~1)
  cat("Not enough clear pixels in this image", "\n")
  Cloud <- idplcd  # all cld
  Shadow[] <- 1-idplcd[]
}   ### Add else statement!!!

# Set working directory
setwd(file.path(OutFolder,imageDirArray[TILE,1]))

# If needed, Write bands 2,3,4,5,10 to file and clear memory
writeRaster(im_B2, filename="im_B2.tif", format="GTiff", overwrite=TRUE); rm(im_B2); gc()
writeRaster(im_B3, filename="im_B3.tif", format="GTiff", overwrite=TRUE); rm(im_B3); gc()
writeRaster(im_B4, filename="im_B4.tif", format="GTiff", overwrite=TRUE); rm(im_B4); gc()
writeRaster(im_B5, filename="im_B5.tif", format="GTiff", overwrite=TRUE); rm(im_B5); gc()
writeRaster(im_B10, filename="im_B10.tif", format="GTiff", overwrite=TRUE); rm(im_B10); gc()

##################################
# Get cloud prob over land

NDSI[((B2Satu[]==1)&(NDSI[]<0))]=0
rm(B2Satu); gc()

NDVI[((B3Satu[]==1)&(NDVI[]>0))]=0
rm(B3Satu); gc()

# Variability probability
Vari_prob <- 1-calc(stack(NDVI,NDSI,whiteness),max)
rm(NDSI,NDVI,whiteness); gc()

# Temperature test
ClearSky = (idlnd[]==1)*1    # Use WT[]==1 or idwt==1 as condition?
Tland = quantile(im_B11[which(ClearSky==1)],probs=c(l_pt,h_pt),na.rm=T)
Temperature_prob = (Tland[2]+400-im_B11[])/(Tland[2]+400-(Tland[1]-400))
rm(ClearSky); gc()

# Final prob mask (land) 
cldprob=20  # 'cldprob' is the cloud probability threshold with default 50 for Landsat 8 (range from 0~100).
final_prob=100*(Vari_prob[]*Temperature_prob) + 100*Thin_prob  # cloud over land probability
clr_max=quantile(final_prob[idlnd[]==1],probs=h_pt,na.rm=T)+cldprob  # dynamic threshold (land)
rm(Vari_prob,Temperature_prob); gc()

##################################   TURN ON LATER if (ptm > 0.1){
# Get cloud prob over water

# Temperature test
ClearSky <- ((idwt==1)&(im_B7[]<=300))*1   # Next three steps replace "which" function
ClearSky[ClearSky[]==0] <- NA
ClearSky <- ClearSky*im_B11[]
Twater <- quantile(ClearSky,probs=h_pt,na.rm=T)
Temperature_prob <- (Twater-im_B11[])/400
rm(Twater,ClearSky); gc()

# Brightness test (over water)
t_bright=1100
Brightness_prob=min(im_B6,t_bright)/t_bright
Brightness_prob[Brightness_prob[]<0]=0
Brightness_prob[Brightness_prob[]>1]=1

# If needed, Write bands 6,7,11 to file and clear memory
writeRaster(im_B6, filename="im_B6.tif", format="GTiff", overwrite=TRUE); rm(im_B6); gc()
writeRaster(im_B7, filename="im_B7.tif", format="GTiff", overwrite=TRUE); rm(im_B7); gc()
writeRaster(im_B11, filename="im_B11.tif", format="GTiff", overwrite=TRUE); rm(im_B11); gc()

# Final prob mask (water)
wfinal_prob=100*(Brightness_prob[]*Temperature_prob)+100*Thin_prob        # cloud over water probability
wclr_max<-quantile(wfinal_prob[idwt==1],probs=h_pt,na.rm=T)+cldprob # dynamic threshold (water) - unsure this is right (GC)
rm(idwt,Brightness_prob,t_bright,Temperature_prob,Thin_prob); gc()

idplcd[]=(idplcd[]==1)&(final_prob>clr_max)&(WT[]==0)|      # cloud over land
  (idplcd[]==1)&(wfinal_prob>wclr_max)&(WT[]==1)                # thin cloud over water
#rm(WT);gc()

plot(idplcd,main="Final Cloud Mask")

# release memory
rm(final_prob,wfinal_prob); gc()

# potential cloud mask\
Cloud[mask[]==1]=0
Cloud[idplcd==1]=1
rm(idplcd); gc()

#######################
# FINAL CLOUD REMOVAL #
#######################

plot(Cloud,main="Final Cloud Layer")

#rm(Cloud,cldprob,clr_max,doy,h_pt,i,   # Also remove Cloud once finalized
#   ptm,raster,reso_ref,reso_thm,resolu,ul,wclr_max,zc)
gc()
