##########################################################################################################
##########################################################################################################
#  Script translated from Fmask Matlab code
#  Converts DNs to TOA ref and BT
#  Add citation for fmask
##########################################################################################################
##########################################################################################################

cat('Reading in Metadata\n')

# Get the metadata file name that should be in the working directory
fileName <- list.files(pattern="MTL.txt")
  
# Read the meatadata file and store lines in linn variable
conn=file(fileName,open="r")
linn=readLines(conn, warn=FALSE)
close(conn)
  
# read in LMAX,LMIN,QCALMAX,QCALMIN,REFMAX,REFMIN
Lmax <- Lmin <- Qcalmax <- Qcalmin <- numeric(length=length(BANDS))
Refmax <- Refmin <- numeric(length=length(BANDS)-2)
for (i in 1:length(BANDS)){
  Lmax[i] = unlist(strsplit(linn[grep(paste0('RADIANCE_MAXIMUM_BAND_',BANDS[i]),linn)],"="))[2]
  Lmin[i] = unlist(strsplit(linn[grep(paste0('RADIANCE_MINIMUM_BAND_',BANDS[i]),linn)],"="))[2]
  Qcalmax[i] = unlist(strsplit(linn[grep(paste0('QUANTIZE_CAL_MAX_BAND_',BANDS[i]),linn)],"="))[2]
  Qcalmin[i] = unlist(strsplit(linn[grep(paste0('QUANTIZE_CAL_MIN_BAND_',BANDS[i]),linn)],"="))[2]
}
Lmax=as.numeric(Lmax); Lmin=as.numeric(Lmin); Qcalmax=as.numeric(Qcalmax); Qcalmin=as.numeric(Qcalmin)

for (i in 1:(length(BANDS)-2)){
  Refmax[i] = unlist(strsplit(linn[grep(paste0('REFLECTANCE_MAXIMUM_BAND_',BANDS[i]),linn)],"="))[2]
  Refmin[i] = unlist(strsplit(linn[grep(paste0('REFLECTANCE_MINIMUM_BAND_',BANDS[i]),linn)],"="))[2]
}
Refmax=as.numeric(Refmax); Refmin=as.numeric(Refmin)
  
# Read in nrows & ncols of optical bands
Sample_ref = unlist(strsplit(linn[grep('REFLECTIVE_SAMPLES',linn)],"="))[2]
Line_ref = unlist(strsplit(linn[grep('REFLECTIVE_LINES',linn)],"="))[2]

# record ijdimension of optical bands
ijdim_ref=as.numeric(c(Line_ref,Sample_ref))
  
# Read in nrows & ncols of thermal bands
Sample_thm = unlist(strsplit(linn[grep('THERMAL_SAMPLES',linn)],"="))[2]
Line_thm = unlist(strsplit(linn[grep('THERMAL_LINES',linn)],"="))[2]

# record thermal band dimensions (i,j)
ijdim_thm = as.numeric(c(Line_thm,Sample_thm))

# Read in resolution of optical and thermal bands
reso_ref = as.numeric(unlist(strsplit(linn[grep('GRID_CELL_SIZE_REFLECTIVE',linn)],"="))[2])
reso_thm = as.numeric(unlist(strsplit(linn[grep('GRID_CELL_SIZE_THERMAL',linn)],"="))[2])

# Read in UTM Zone Number
zc = as.numeric(unlist(strsplit(linn[grep('UTM_ZONE',linn)],"="))[2])
  
# Read in Solar Azimuth & Elevation angle (degrees)
azi = as.numeric(unlist(strsplit(linn[grep('SUN_AZIMUTH',linn)],"="))[2])
zen = 90-as.numeric(unlist(strsplit(linn[grep('SUN_ELEVATION',linn)],"="))[2])

# Read in upperleft mapx,y
ulx = unlist(strsplit(linn[grep('CORNER_UL_PROJECTION_X_PRODUCT',linn)],"="))[2]
uly = unlist(strsplit(linn[grep('CORNER_UL_PROJECTION_Y_PRODUCT',linn)],"="))[2]
ul = as.numeric(c(ulx,uly))

# Read in date of year
doy=as.integer(substr(fileName,14,16))

# Delete extra objects
rm(list= ls()[!(ls() %in% c('Lmax','Lmin','Refmax','Refmin','OutFolder','Qcalmax','Qcalmin','ijdim_ref','gcr','tmpDIR',
                            'reso_ref','reso_thm','ul','zen','azi','zc','Lnum','doy','ScriptFolder','DEMFileName',
                            'baseDir','imageDirArray','BANDS','TILE','TrainingFileName'))])

#########################################################################################################
# LPGS Upper lef corner alignment (see Landsat handbook for detail)
ul[1]=ul[1]-15
ul[2]=ul[2]+15
resolu=c(reso_ref,reso_ref)

# Read in all bands
for(i in BANDS){
  raster <- paste0(imageDirArray[TILE,3],"_B",i,".TIF")
  assign(paste0("im","_B",i),raster(raster))
}

# check to see whether need to resample thermal band - check nd2toa script or ask Ned for help
if (reso_ref!=reso_thm){
  print("Need to resample thermal band?") 
  breakerr()
}

# only processing pixels where all bands have values (id_missing)
id_missing<-((im_B2[]==0)|(im_B3[]==0)|(im_B4[]==0)|(im_B5[]==0)|(im_B6[]==0)|
               (im_B7[]==0)|(im_B9[]==0)|(im_B10[]==0)|(im_B11[]==0))

# find pixels that are saturated in the visible bands
B1Satu=im_B2[]==65535; B2Satu=im_B3[]==65535; B3Satu=im_B4[]==65535;

# ND to TOA reflectance with 0.0001 scale_facor
cat('From DNs to TOA ref & BT\n')
for(i in 1:7){
  raster <- get(paste0("im","_B",BANDS[i]))
  raster[] <- ((Refmax[i]-Refmin[i])/(Qcalmax[i]-Qcalmin[i]))*(raster[]-Qcalmin[i])+Refmin[i]
  assign(paste0("im","_B",BANDS[i]),raster)
}

im_B10[] <- ((Lmax[8]-Lmin[8])/(Qcalmax[8]-Qcalmin[8]))*(im_B10[]-Qcalmin[8])+Lmin[8];
im_B11[] <- ((Lmax[9]-Lmin[9])/(Qcalmax[9]-Qcalmin[9]))*(im_B11[]-Qcalmin[9])+Lmin[9];

rm(Lmax,Lmin,Qcalmin,Qcalmax,Refmin,Refmax)

# compute TOA reflectances with a correction for the sun angle
s_zen=zen*3.141593/180  # convert to radians
for(i in 1:7){
  tmp <- get(paste0("im","_B",BANDS[i]))
  tmp[] <- 10000*tmp[]/cos(s_zen)
  assign(paste0("im","_B",BANDS[i]),tmp)
}
rm(tmp)

# convert Band6 from radiance to BT
cat('Converting From Band 6 Radiance to Brightness Temperature\n')
K1_B10 =  774.89; K2_B10 = 1321.08;
K1_B11 =  480.89; K2_B11 = 1201.14;

im_B10[]=K2_B10/log((K1_B10/im_B10[])+1)
im_B11[]=K2_B11/log((K1_B11/im_B11[])+1)

rm(K1_B10,K1_B11,K2_B10,K2_B11)

# convert from Kelvin to Celcius with 0.01 scale_facor
im_B10[]=100*(im_B10[]-273.15);
im_B11[]=100*(im_B11[]-273.15);

# get data ready for Fmask
im_B2[id_missing==1]=NA; im_B3[id_missing==1]=NA; im_B4[id_missing==1]=NA; im_B5[id_missing==1]=NA;
im_B6[id_missing==1]=NA; im_B7[id_missing==1]=NA; im_B9[id_missing==1]=NA;

im_B10[id_missing==1] = NA;
im_B11[id_missing==1] = NA;

rm(id_missing,raster)
gc()
