##########################################################################################################
##########################################################################################################
#  Script resamples DEM to same extent as landsat imagery
#  Calculates slope and aspect
#  and performs c-correction on each landsat band
##########################################################################################################
##########################################################################################################

cat("Calculating Illumination","\n")
setwd(file.path(OutFolder,imageDirArray[TILE,1]))  # Where to save topo-corrected rasters
DEM <- raster(DEMFileName)   # Import DEM
DEM <- crop(DEM,raster(paste0("im_B2.tif")))   # Crop to extent of current landsat tile
DEM <- resample(DEM,raster(paste0("im_B2.tif")),method="bilinear")   # Resample to align DEM w/ landsat

# Overwrite DEM w/ calculated slope and aspect
DEM <- stack(terrain(DEM, opt=c('slope'), unit='radians'), terrain(DEM, opt=c('aspect'), unit='radians'))

# Convert solar azimuth to radians
azi_rad <- azi*(pi/180)

# Calculate illumination
illumination <- cos(DEM[[1]])*cos(s_zen) + sin(DEM[[1]])*sin(s_zen)*cos(azi_rad - DEM[[2]])

# Loop over bands and correct
for(i in 2:7){
  cat(paste0("Correcting Raster im","_B",i,"..."),"\n")
  tofix <- raster(paste0("im","_B",i,".tif"))
  C <- lm(tofix[] ~ illumination[])
  C <- coefficients(C)[[1]]/coefficients(C)[[2]]
  tofix[] <- tofix[]*(cos(s_zen)+C)/(illumination[]+C)
  writeRaster(tofix, filename=paste0("im","_B",i,".tif"), format="GTiff", overwrite=TRUE)   # write that raster to file
}

rm(tofix,illumination,C,DEM)
gc()
